import smtplib
from email.mime.text import MIMEText

from inni.modules.base import BaseModule
from inni.template import render_template


class Module(BaseModule):
    login_template_keys = ("login_subject", "login_body")
    logout_template_keys = ("logout_subject", "logout_body")

    def setUp(self):
        if self.config["ssl"]:
            self.mailer = smtplib.SMTP_SSL(self.config["host"], self.config["port"])
        else:
            self.mailer = smtplib.SMTP(self.config["host"], self.config["port"])

    def send_mail(self, sub, body):
        msg = MIMEText(body)
        msg["Subject"] = sub
        from_ = msg["From"] = self.config.get("from", self.config["username"])
        to = msg["To"] = ", ".join(self.config["to"])
        with self.mailer as smtp:
            smtp.login(self.config["username"], self.config["password"])
            smtp.sendmail(from_, to, msg.as_string())

    def login(self, responses):
        body = render_template(self.config["login_body"], **responses)
        subject = render_template(self.config["login_subject"], **responses)
        with self.out.status("[green]Sending Email"):
            self.send_mail(subject, body)
        self.out.print("[green]✅ Email sent")

    def logout(self, responses):
        body = render_template(self.config["logout_body"], **responses)
        subject = render_template(self.config["logout_subject"], **responses)
        with self.out.status("[green]Sending Email"):
            self.send_mail(subject, body)
        self.out.print("[green]✅ Email sent")



from rich.console import Console

from inni.template import template_to_vars


class BaseModule:
    login_template_keys = ()
    logout_template_keys = ()

    def __init__(self, config: dict, out: Console, err: Console):
        self.config = config
        self.out = out
        self.err = err
        self.setUp()

    def setUp(self):
        pass

    def template_variables(self) -> dict[str, tuple[str, ...]]:
        """
        Returns the list of variables that the module expects
        """

        def keys_to_vars(keys):
            return tuple(
                var
                for key in keys
                for var in template_to_vars(self.config.get(key, ""))
            )

        return {
            "login": keys_to_vars(self.login_template_keys),
            "logout": keys_to_vars(self.logout_template_keys),
        }

    def login(self, responses: dict):
        raise NotImplementedError()

    def logout(self, responses: dict):
        raise NotImplementedError()



from skpy import Skype

from inni.modules.base import BaseModule
from inni.template import render_template


class Module(BaseModule):
    login_template_keys = ("login_template",)
    logout_template_keys = ("logout_template",)

    def setUp(self):
        self.sk = Skype(self.config["username"], self.config["password"])
        self.chat = self.sk.chats[self.config["chat_id"]]

    def send_message(self, msg):
        rich = self.config.get("rich", False)
        self.chat.sendMsg(msg, rich=rich)

    def login(self, responses):
        msg = render_template(self.config["login_template"], **responses)
        with self.out.status("[green]Sending Skype Message"):
            self.send_message(msg)
        self.out.print("[green]✅ Skype message sent")

    def logout(self, responses):
        msg = render_template(self.config["logout_template"], **responses)
        with self.out.status("[green]Sending Skype Message"):
            self.send_message(msg)
        self.out.print("[green]✅ Skype message sent")



import re

from jira import JIRA, Issue, JIRAError
from prompt_toolkit import PromptSession
from prompt_toolkit.completion import FuzzyWordCompleter
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.validation import ValidationError, Validator
from rich.table import Table

from inni.modules.base import BaseModule

TICKET_RE = re.compile(r"\w+-\d+")


class WorkTimeValidator(Validator):
    regex = re.compile(r"(\d+w\s*)?(\d+d\s*)?(\d+h\s*)?(\d+s\s*)?")

    def validate(self, document):
        if not self.regex.fullmatch(document.text):
            raise ValidationError(
                0,
                "Use the format: 2w 4d 6h 45m."
                " w = weeks, d = days, h = hours, m = minutes",
            )


class Module(BaseModule):
    def setUp(self):
        self.jira = JIRA(
            self.config["instance_url"],
            basic_auth=(self.config["username"], self.config["password"]),
        )

    def print_issues(self, issues: list[Issue], title="Jira Tickets"):
        table = Table(title=title)
        table.add_column("ID", style="cyan")
        table.add_column("Name", style="magenta")
        table.add_column("Creator", style="green")
        table.add_column("Status", style="cyan")
        table.add_column("Priority", style="blue")

        for issue in issues:
            table.add_row(
                issue.key,
                issue.fields.summary,
                issue.fields.creator.displayName,
                issue.fields.status.name,
                issue.fields.priority.name,
            )
        self.out.print(table)

    def update_workhours(self):
        issue_session = PromptSession()
        worktime_session = PromptSession(validator=WorkTimeValidator())

        def blue(text):
            return FormattedText([("ansiblue", text)])

        jql = self.config.get("jql", "assignee = currentUser()")
        search = True
        keys = []
        while True:
            if search:
                with self.out.status(f"[blue]Searching issues: {jql}"):
                    issues: list[Issue] = self.jira.search_issues(jql)
                    self.print_issues(issues, jql)
                    keys = [issue.key for issue in issues]
                search = False
            self.out.print(
                "[blue]Select issue to update. Leave blank to finish."
                " To enter a new search, prefix with ?"
            )
            response = issue_session.prompt(
                blue("> "), completer=FuzzyWordCompleter(keys)
            )
            if not response:
                break
            elif response.startswith("?"):
                jql = response[1:]
                search = True
            elif TICKET_RE.fullmatch(response):
                try:
                    self.jira.issue(response)
                except JIRAError:
                    self.err.print(
                        "[red]Error fetching issue {response}."
                        " Most likely, it either doesn't exist"
                    )

                time = worktime_session.prompt(
                    blue(f"Enter time to add to {response}: ")
                )
                if not time:
                    self.out.print(f"[yellow]Skipping adding time to {response}")
                    continue
                self.jira.add_worklog(response, time)
                self.out.print("[green]Time Added!")
            else:
                self.err.print("[red]Invalid Response")

    def login(self, responses):
        self.update_workhours()

    def logout(self, responses):
        self.update_workhours()



import re
import subprocess

import psutil

from inni.modules.base import BaseModule


class Module(BaseModule):
    login_template_keys = ("login_template",)
    logout_template_keys = ("logout_template",)

    def handle_entry(self, entry):
        if "exec" not in entry:
            name = entry.get("name", "unnamed")
            self.err.print(f"No exec found in {name} entry. Skipping.")
            return

        name = entry.get("name", entry["exec"])
        shell = entry.get("shell", False)

        self.out.print(f"[bold]Entry: {name}")
        if "if_running" in entry:
            regex = re.compile(entry["if_running"])
            for process in self.processes:
                if regex.findall(process):
                    self.out.print("[green]✅ Matching process found.")
                    break
            else:
                self.out.print(
                    "[yellow]✗  No process matching regex found. Skipping entry"
                )
                return

        if "if_not_running" in entry:
            regex = re.compile(entry["if_not_running"])
            for process in self.processes:
                if regex.findall(process):
                    self.out.print(
                        "[yellow]✗  Process matching regex found. Skipping entry"
                    )
                    return
            self.out.print("[green]✅ No such process is running.")

        if "if_true" in entry:
            self.out.print(f"[blue]Running {entry['if_true']}")
            process = subprocess.run(
                entry["if_true"],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
                shell=shell,
            )
            if process.returncode:
                self.out.print(
                    f"[yellow]✗  Process exited with exit code {process.returncode}. "
                    f"Skipping entry."
                )
                return
            self.out.print("[green]✅ Check process ran as expected.")

        if "if_false" in entry:
            self.out.print(f"[blue]Running {entry['if_false']}")
            process = subprocess.run(
                entry["if_false"],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
                shell=shell,
            )
            if not process.returncode:
                self.out.print("[yellow]✗  Process ran successfully. Skipping entry")
                return
            self.out.print("[green]✅ Check process failed as expected.")

        self.out.print("[green]✅ Running entry")
        subprocess.Popen(
            entry["exec"],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
            shell=shell,
        )

    def run_all(self, login: bool):
        entries = self.config.get("login" if login else "logout", [])
        self.processes = [
            " ".join(i.cmdline()) or i.name() for i in psutil.process_iter()
        ]
        for entry in entries:
            self.handle_entry(entry)
            self.out.print()

    def login(self, responses):
        self.run_all(True)

    def logout(self, responses):
        self.run_all(False)
